
''' Generated by OTF2 Template Engine '''

import ctypes

from .Config import conf, StrParam
from .ErrorCodes import ErrorCode, HandleErrorCode
from .GeneralDefinitions import *
from .AttributeList import AttributeList


class EventSizeEstimator(ctypes.Structure):
    pass

def EventSizeEstimator_New():
    c_New = conf.lib.OTF2_EventSizeEstimator_New
    c_New.argtypes = []
    c_New.restype = ctypes.POINTER(EventSizeEstimator)
    # NOTE: Do not errcheck here. This function returns a value, not an error code
    return c_EventSizeEstimator_New()

def EventSizeEstimator_Delete(estimator):
    c_Delete = conf.lib.OTF2_EventSizeEstimator_Delete
    c_Delete.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_Delete.restype = ErrorCode
    c_Delete.errcheck = HandleErrorCode
    c_Delete(estimator)

def EventSizeEstimator_SetNumberOfStringDefinitions(estimator, numberOfStringDefinitions):
    c_SetNumberOfStringDefinitions = conf.lib.OTF2_EventSizeEstimator_SetNumberOfStringDefinitions
    c_SetNumberOfStringDefinitions.argtypes = [ ctypes.POINTER(EventSizeEstimator), StringRef ]
    c_SetNumberOfStringDefinitions.restype = ErrorCode
    c_SetNumberOfStringDefinitions.errcheck = HandleErrorCode
    c_SetNumberOfStringDefinitions(estimator, numberOfStringDefinitions)

def EventSizeEstimator_SetNumberOfAttributeDefinitions(estimator, numberOfAttributeDefinitions):
    c_SetNumberOfAttributeDefinitions = conf.lib.OTF2_EventSizeEstimator_SetNumberOfAttributeDefinitions
    c_SetNumberOfAttributeDefinitions.argtypes = [ ctypes.POINTER(EventSizeEstimator), AttributeRef ]
    c_SetNumberOfAttributeDefinitions.restype = ErrorCode
    c_SetNumberOfAttributeDefinitions.errcheck = HandleErrorCode
    c_SetNumberOfAttributeDefinitions(estimator, numberOfAttributeDefinitions)

def EventSizeEstimator_SetNumberOfLocationDefinitions(estimator, numberOfLocationDefinitions):
    c_SetNumberOfLocationDefinitions = conf.lib.OTF2_EventSizeEstimator_SetNumberOfLocationDefinitions
    c_SetNumberOfLocationDefinitions.argtypes = [ ctypes.POINTER(EventSizeEstimator), LocationRef ]
    c_SetNumberOfLocationDefinitions.restype = ErrorCode
    c_SetNumberOfLocationDefinitions.errcheck = HandleErrorCode
    c_SetNumberOfLocationDefinitions(estimator, numberOfLocationDefinitions)

def EventSizeEstimator_SetNumberOfRegionDefinitions(estimator, numberOfRegionDefinitions):
    c_SetNumberOfRegionDefinitions = conf.lib.OTF2_EventSizeEstimator_SetNumberOfRegionDefinitions
    c_SetNumberOfRegionDefinitions.argtypes = [ ctypes.POINTER(EventSizeEstimator), RegionRef ]
    c_SetNumberOfRegionDefinitions.restype = ErrorCode
    c_SetNumberOfRegionDefinitions.errcheck = HandleErrorCode
    c_SetNumberOfRegionDefinitions(estimator, numberOfRegionDefinitions)

def EventSizeEstimator_SetNumberOfGroupDefinitions(estimator, numberOfGroupDefinitions):
    c_SetNumberOfGroupDefinitions = conf.lib.OTF2_EventSizeEstimator_SetNumberOfGroupDefinitions
    c_SetNumberOfGroupDefinitions.argtypes = [ ctypes.POINTER(EventSizeEstimator), GroupRef ]
    c_SetNumberOfGroupDefinitions.restype = ErrorCode
    c_SetNumberOfGroupDefinitions.errcheck = HandleErrorCode
    c_SetNumberOfGroupDefinitions(estimator, numberOfGroupDefinitions)

def EventSizeEstimator_SetNumberOfMetricDefinitions(estimator, numberOfMetricDefinitions):
    c_SetNumberOfMetricDefinitions = conf.lib.OTF2_EventSizeEstimator_SetNumberOfMetricDefinitions
    c_SetNumberOfMetricDefinitions.argtypes = [ ctypes.POINTER(EventSizeEstimator), MetricRef ]
    c_SetNumberOfMetricDefinitions.restype = ErrorCode
    c_SetNumberOfMetricDefinitions.errcheck = HandleErrorCode
    c_SetNumberOfMetricDefinitions(estimator, numberOfMetricDefinitions)

def EventSizeEstimator_SetNumberOfCommDefinitions(estimator, numberOfCommDefinitions):
    c_SetNumberOfCommDefinitions = conf.lib.OTF2_EventSizeEstimator_SetNumberOfCommDefinitions
    c_SetNumberOfCommDefinitions.argtypes = [ ctypes.POINTER(EventSizeEstimator), CommRef ]
    c_SetNumberOfCommDefinitions.restype = ErrorCode
    c_SetNumberOfCommDefinitions.errcheck = HandleErrorCode
    c_SetNumberOfCommDefinitions(estimator, numberOfCommDefinitions)

def EventSizeEstimator_SetNumberOfParameterDefinitions(estimator, numberOfParameterDefinitions):
    c_SetNumberOfParameterDefinitions = conf.lib.OTF2_EventSizeEstimator_SetNumberOfParameterDefinitions
    c_SetNumberOfParameterDefinitions.argtypes = [ ctypes.POINTER(EventSizeEstimator), ParameterRef ]
    c_SetNumberOfParameterDefinitions.restype = ErrorCode
    c_SetNumberOfParameterDefinitions.errcheck = HandleErrorCode
    c_SetNumberOfParameterDefinitions(estimator, numberOfParameterDefinitions)

def EventSizeEstimator_SetNumberOfRmaWinDefinitions(estimator, numberOfRmaWinDefinitions):
    c_SetNumberOfRmaWinDefinitions = conf.lib.OTF2_EventSizeEstimator_SetNumberOfRmaWinDefinitions
    c_SetNumberOfRmaWinDefinitions.argtypes = [ ctypes.POINTER(EventSizeEstimator), RmaWinRef ]
    c_SetNumberOfRmaWinDefinitions.restype = ErrorCode
    c_SetNumberOfRmaWinDefinitions.errcheck = HandleErrorCode
    c_SetNumberOfRmaWinDefinitions(estimator, numberOfRmaWinDefinitions)

def EventSizeEstimator_SetNumberOfSourceCodeLocationDefinitions(estimator, numberOfSourceCodeLocationDefinitions):
    c_SetNumberOfSourceCodeLocationDefinitions = conf.lib.OTF2_EventSizeEstimator_SetNumberOfSourceCodeLocationDefinitions
    c_SetNumberOfSourceCodeLocationDefinitions.argtypes = [ ctypes.POINTER(EventSizeEstimator), SourceCodeLocationRef ]
    c_SetNumberOfSourceCodeLocationDefinitions.restype = ErrorCode
    c_SetNumberOfSourceCodeLocationDefinitions.errcheck = HandleErrorCode
    c_SetNumberOfSourceCodeLocationDefinitions(estimator, numberOfSourceCodeLocationDefinitions)

def EventSizeEstimator_SetNumberOfCallingContextDefinitions(estimator, numberOfCallingContextDefinitions):
    c_SetNumberOfCallingContextDefinitions = conf.lib.OTF2_EventSizeEstimator_SetNumberOfCallingContextDefinitions
    c_SetNumberOfCallingContextDefinitions.argtypes = [ ctypes.POINTER(EventSizeEstimator), CallingContextRef ]
    c_SetNumberOfCallingContextDefinitions.restype = ErrorCode
    c_SetNumberOfCallingContextDefinitions.errcheck = HandleErrorCode
    c_SetNumberOfCallingContextDefinitions(estimator, numberOfCallingContextDefinitions)

def EventSizeEstimator_SetNumberOfInterruptGeneratorDefinitions(estimator, numberOfInterruptGeneratorDefinitions):
    c_SetNumberOfInterruptGeneratorDefinitions = conf.lib.OTF2_EventSizeEstimator_SetNumberOfInterruptGeneratorDefinitions
    c_SetNumberOfInterruptGeneratorDefinitions.argtypes = [ ctypes.POINTER(EventSizeEstimator), InterruptGeneratorRef ]
    c_SetNumberOfInterruptGeneratorDefinitions.restype = ErrorCode
    c_SetNumberOfInterruptGeneratorDefinitions.errcheck = HandleErrorCode
    c_SetNumberOfInterruptGeneratorDefinitions(estimator, numberOfInterruptGeneratorDefinitions)

def EventSizeEstimator_SetNumberOfIoFileDefinitions(estimator, numberOfIoFileDefinitions):
    c_SetNumberOfIoFileDefinitions = conf.lib.OTF2_EventSizeEstimator_SetNumberOfIoFileDefinitions
    c_SetNumberOfIoFileDefinitions.argtypes = [ ctypes.POINTER(EventSizeEstimator), IoFileRef ]
    c_SetNumberOfIoFileDefinitions.restype = ErrorCode
    c_SetNumberOfIoFileDefinitions.errcheck = HandleErrorCode
    c_SetNumberOfIoFileDefinitions(estimator, numberOfIoFileDefinitions)

def EventSizeEstimator_SetNumberOfIoHandleDefinitions(estimator, numberOfIoHandleDefinitions):
    c_SetNumberOfIoHandleDefinitions = conf.lib.OTF2_EventSizeEstimator_SetNumberOfIoHandleDefinitions
    c_SetNumberOfIoHandleDefinitions.argtypes = [ ctypes.POINTER(EventSizeEstimator), IoHandleRef ]
    c_SetNumberOfIoHandleDefinitions.restype = ErrorCode
    c_SetNumberOfIoHandleDefinitions.errcheck = HandleErrorCode
    c_SetNumberOfIoHandleDefinitions(estimator, numberOfIoHandleDefinitions)

def EventSizeEstimator_SetNumberOfLocationGroupDefinitions(estimator, numberOfLocationGroupDefinitions):
    c_SetNumberOfLocationGroupDefinitions = conf.lib.OTF2_EventSizeEstimator_SetNumberOfLocationGroupDefinitions
    c_SetNumberOfLocationGroupDefinitions.argtypes = [ ctypes.POINTER(EventSizeEstimator), LocationGroupRef ]
    c_SetNumberOfLocationGroupDefinitions.restype = ErrorCode
    c_SetNumberOfLocationGroupDefinitions.errcheck = HandleErrorCode
    c_SetNumberOfLocationGroupDefinitions(estimator, numberOfLocationGroupDefinitions)

def EventSizeEstimator_GetDefChunkSize(estimator):
    c_GetDefChunkSize = conf.lib.OTF2_EventSizeEstimator_GetDefChunkSize
    c_GetDefChunkSize.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetDefChunkSize.restype = ctypes.c_size_t
    return c_GetDefChunkSize(estimator)

def EventSizeEstimator_GetSizeOfTimestamp(estimator):
    c_GetSizeOfTimestamp = conf.lib.OTF2_EventSizeEstimator_GetSizeOfTimestamp
    c_GetSizeOfTimestamp.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfTimestamp.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfTimestamp(estimator)

def EventSizeEstimator_GetSizeOfAttributeList(estimator, attributeList):
    c_GetSizeOfAttributeList = conf.lib.OTF2_EventSizeEstimator_GetSizeOfAttributeList
    c_GetSizeOfAttributeList.argtypes = [ ctypes.POINTER(EventSizeEstimator), ctypes.POINTER(AttributeList) ]
    c_GetSizeOfAttributeList.restype = ctypes.c_size_t
    return c_GetSizeOfAttributeList(estimator, attributeList)

def EventSizeEstimator_GetSizeOfBufferFlushEvent(estimator):
    c_GetSizeOfBufferFlushEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfBufferFlushEvent
    c_GetSizeOfBufferFlushEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfBufferFlushEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfBufferFlushEvent(estimator)

def EventSizeEstimator_GetSizeOfMeasurementOnOffEvent(estimator):
    c_GetSizeOfMeasurementOnOffEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfMeasurementOnOffEvent
    c_GetSizeOfMeasurementOnOffEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfMeasurementOnOffEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfMeasurementOnOffEvent(estimator)

def EventSizeEstimator_GetSizeOfEnterEvent(estimator):
    c_GetSizeOfEnterEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfEnterEvent
    c_GetSizeOfEnterEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfEnterEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfEnterEvent(estimator)

def EventSizeEstimator_GetSizeOfLeaveEvent(estimator):
    c_GetSizeOfLeaveEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfLeaveEvent
    c_GetSizeOfLeaveEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfLeaveEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfLeaveEvent(estimator)

def EventSizeEstimator_GetSizeOfMpiSendEvent(estimator):
    c_GetSizeOfMpiSendEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfMpiSendEvent
    c_GetSizeOfMpiSendEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfMpiSendEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfMpiSendEvent(estimator)

def EventSizeEstimator_GetSizeOfMpiIsendEvent(estimator):
    c_GetSizeOfMpiIsendEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfMpiIsendEvent
    c_GetSizeOfMpiIsendEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfMpiIsendEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfMpiIsendEvent(estimator)

def EventSizeEstimator_GetSizeOfMpiIsendCompleteEvent(estimator):
    c_GetSizeOfMpiIsendCompleteEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfMpiIsendCompleteEvent
    c_GetSizeOfMpiIsendCompleteEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfMpiIsendCompleteEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfMpiIsendCompleteEvent(estimator)

def EventSizeEstimator_GetSizeOfMpiIrecvRequestEvent(estimator):
    c_GetSizeOfMpiIrecvRequestEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfMpiIrecvRequestEvent
    c_GetSizeOfMpiIrecvRequestEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfMpiIrecvRequestEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfMpiIrecvRequestEvent(estimator)

def EventSizeEstimator_GetSizeOfMpiRecvEvent(estimator):
    c_GetSizeOfMpiRecvEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfMpiRecvEvent
    c_GetSizeOfMpiRecvEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfMpiRecvEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfMpiRecvEvent(estimator)

def EventSizeEstimator_GetSizeOfMpiIrecvEvent(estimator):
    c_GetSizeOfMpiIrecvEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfMpiIrecvEvent
    c_GetSizeOfMpiIrecvEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfMpiIrecvEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfMpiIrecvEvent(estimator)

def EventSizeEstimator_GetSizeOfMpiRequestTestEvent(estimator):
    c_GetSizeOfMpiRequestTestEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfMpiRequestTestEvent
    c_GetSizeOfMpiRequestTestEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfMpiRequestTestEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfMpiRequestTestEvent(estimator)

def EventSizeEstimator_GetSizeOfMpiRequestCancelledEvent(estimator):
    c_GetSizeOfMpiRequestCancelledEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfMpiRequestCancelledEvent
    c_GetSizeOfMpiRequestCancelledEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfMpiRequestCancelledEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfMpiRequestCancelledEvent(estimator)

def EventSizeEstimator_GetSizeOfMpiCollectiveBeginEvent(estimator):
    c_GetSizeOfMpiCollectiveBeginEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfMpiCollectiveBeginEvent
    c_GetSizeOfMpiCollectiveBeginEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfMpiCollectiveBeginEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfMpiCollectiveBeginEvent(estimator)

def EventSizeEstimator_GetSizeOfMpiCollectiveEndEvent(estimator):
    c_GetSizeOfMpiCollectiveEndEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfMpiCollectiveEndEvent
    c_GetSizeOfMpiCollectiveEndEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfMpiCollectiveEndEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfMpiCollectiveEndEvent(estimator)

def EventSizeEstimator_GetSizeOfOmpForkEvent(estimator):
    c_GetSizeOfOmpForkEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfOmpForkEvent
    c_GetSizeOfOmpForkEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfOmpForkEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfOmpForkEvent(estimator)

def EventSizeEstimator_GetSizeOfOmpJoinEvent(estimator):
    c_GetSizeOfOmpJoinEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfOmpJoinEvent
    c_GetSizeOfOmpJoinEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfOmpJoinEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfOmpJoinEvent(estimator)

def EventSizeEstimator_GetSizeOfOmpAcquireLockEvent(estimator):
    c_GetSizeOfOmpAcquireLockEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfOmpAcquireLockEvent
    c_GetSizeOfOmpAcquireLockEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfOmpAcquireLockEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfOmpAcquireLockEvent(estimator)

def EventSizeEstimator_GetSizeOfOmpReleaseLockEvent(estimator):
    c_GetSizeOfOmpReleaseLockEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfOmpReleaseLockEvent
    c_GetSizeOfOmpReleaseLockEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfOmpReleaseLockEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfOmpReleaseLockEvent(estimator)

def EventSizeEstimator_GetSizeOfOmpTaskCreateEvent(estimator):
    c_GetSizeOfOmpTaskCreateEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfOmpTaskCreateEvent
    c_GetSizeOfOmpTaskCreateEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfOmpTaskCreateEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfOmpTaskCreateEvent(estimator)

def EventSizeEstimator_GetSizeOfOmpTaskSwitchEvent(estimator):
    c_GetSizeOfOmpTaskSwitchEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfOmpTaskSwitchEvent
    c_GetSizeOfOmpTaskSwitchEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfOmpTaskSwitchEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfOmpTaskSwitchEvent(estimator)

def EventSizeEstimator_GetSizeOfOmpTaskCompleteEvent(estimator):
    c_GetSizeOfOmpTaskCompleteEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfOmpTaskCompleteEvent
    c_GetSizeOfOmpTaskCompleteEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfOmpTaskCompleteEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfOmpTaskCompleteEvent(estimator)

def EventSizeEstimator_GetSizeOfMetricEvent(estimator, numberOfMetrics):
    c_GetSizeOfMetricEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfMetricEvent
    c_GetSizeOfMetricEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator), ctypes.c_uint8 ]
    c_GetSizeOfMetricEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfMetricEvent(estimator, numberOfMetrics)

def EventSizeEstimator_GetSizeOfParameterStringEvent(estimator):
    c_GetSizeOfParameterStringEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfParameterStringEvent
    c_GetSizeOfParameterStringEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfParameterStringEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfParameterStringEvent(estimator)

def EventSizeEstimator_GetSizeOfParameterIntEvent(estimator):
    c_GetSizeOfParameterIntEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfParameterIntEvent
    c_GetSizeOfParameterIntEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfParameterIntEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfParameterIntEvent(estimator)

def EventSizeEstimator_GetSizeOfParameterUnsignedIntEvent(estimator):
    c_GetSizeOfParameterUnsignedIntEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfParameterUnsignedIntEvent
    c_GetSizeOfParameterUnsignedIntEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfParameterUnsignedIntEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfParameterUnsignedIntEvent(estimator)

def EventSizeEstimator_GetSizeOfRmaWinCreateEvent(estimator):
    c_GetSizeOfRmaWinCreateEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfRmaWinCreateEvent
    c_GetSizeOfRmaWinCreateEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfRmaWinCreateEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfRmaWinCreateEvent(estimator)

def EventSizeEstimator_GetSizeOfRmaWinDestroyEvent(estimator):
    c_GetSizeOfRmaWinDestroyEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfRmaWinDestroyEvent
    c_GetSizeOfRmaWinDestroyEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfRmaWinDestroyEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfRmaWinDestroyEvent(estimator)

def EventSizeEstimator_GetSizeOfRmaCollectiveBeginEvent(estimator):
    c_GetSizeOfRmaCollectiveBeginEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfRmaCollectiveBeginEvent
    c_GetSizeOfRmaCollectiveBeginEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfRmaCollectiveBeginEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfRmaCollectiveBeginEvent(estimator)

def EventSizeEstimator_GetSizeOfRmaCollectiveEndEvent(estimator):
    c_GetSizeOfRmaCollectiveEndEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfRmaCollectiveEndEvent
    c_GetSizeOfRmaCollectiveEndEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfRmaCollectiveEndEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfRmaCollectiveEndEvent(estimator)

def EventSizeEstimator_GetSizeOfRmaGroupSyncEvent(estimator):
    c_GetSizeOfRmaGroupSyncEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfRmaGroupSyncEvent
    c_GetSizeOfRmaGroupSyncEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfRmaGroupSyncEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfRmaGroupSyncEvent(estimator)

def EventSizeEstimator_GetSizeOfRmaRequestLockEvent(estimator):
    c_GetSizeOfRmaRequestLockEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfRmaRequestLockEvent
    c_GetSizeOfRmaRequestLockEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfRmaRequestLockEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfRmaRequestLockEvent(estimator)

def EventSizeEstimator_GetSizeOfRmaAcquireLockEvent(estimator):
    c_GetSizeOfRmaAcquireLockEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfRmaAcquireLockEvent
    c_GetSizeOfRmaAcquireLockEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfRmaAcquireLockEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfRmaAcquireLockEvent(estimator)

def EventSizeEstimator_GetSizeOfRmaTryLockEvent(estimator):
    c_GetSizeOfRmaTryLockEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfRmaTryLockEvent
    c_GetSizeOfRmaTryLockEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfRmaTryLockEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfRmaTryLockEvent(estimator)

def EventSizeEstimator_GetSizeOfRmaReleaseLockEvent(estimator):
    c_GetSizeOfRmaReleaseLockEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfRmaReleaseLockEvent
    c_GetSizeOfRmaReleaseLockEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfRmaReleaseLockEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfRmaReleaseLockEvent(estimator)

def EventSizeEstimator_GetSizeOfRmaSyncEvent(estimator):
    c_GetSizeOfRmaSyncEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfRmaSyncEvent
    c_GetSizeOfRmaSyncEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfRmaSyncEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfRmaSyncEvent(estimator)

def EventSizeEstimator_GetSizeOfRmaWaitChangeEvent(estimator):
    c_GetSizeOfRmaWaitChangeEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfRmaWaitChangeEvent
    c_GetSizeOfRmaWaitChangeEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfRmaWaitChangeEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfRmaWaitChangeEvent(estimator)

def EventSizeEstimator_GetSizeOfRmaPutEvent(estimator):
    c_GetSizeOfRmaPutEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfRmaPutEvent
    c_GetSizeOfRmaPutEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfRmaPutEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfRmaPutEvent(estimator)

def EventSizeEstimator_GetSizeOfRmaGetEvent(estimator):
    c_GetSizeOfRmaGetEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfRmaGetEvent
    c_GetSizeOfRmaGetEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfRmaGetEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfRmaGetEvent(estimator)

def EventSizeEstimator_GetSizeOfRmaAtomicEvent(estimator):
    c_GetSizeOfRmaAtomicEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfRmaAtomicEvent
    c_GetSizeOfRmaAtomicEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfRmaAtomicEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfRmaAtomicEvent(estimator)

def EventSizeEstimator_GetSizeOfRmaOpCompleteBlockingEvent(estimator):
    c_GetSizeOfRmaOpCompleteBlockingEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfRmaOpCompleteBlockingEvent
    c_GetSizeOfRmaOpCompleteBlockingEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfRmaOpCompleteBlockingEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfRmaOpCompleteBlockingEvent(estimator)

def EventSizeEstimator_GetSizeOfRmaOpCompleteNonBlockingEvent(estimator):
    c_GetSizeOfRmaOpCompleteNonBlockingEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfRmaOpCompleteNonBlockingEvent
    c_GetSizeOfRmaOpCompleteNonBlockingEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfRmaOpCompleteNonBlockingEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfRmaOpCompleteNonBlockingEvent(estimator)

def EventSizeEstimator_GetSizeOfRmaOpTestEvent(estimator):
    c_GetSizeOfRmaOpTestEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfRmaOpTestEvent
    c_GetSizeOfRmaOpTestEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfRmaOpTestEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfRmaOpTestEvent(estimator)

def EventSizeEstimator_GetSizeOfRmaOpCompleteRemoteEvent(estimator):
    c_GetSizeOfRmaOpCompleteRemoteEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfRmaOpCompleteRemoteEvent
    c_GetSizeOfRmaOpCompleteRemoteEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfRmaOpCompleteRemoteEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfRmaOpCompleteRemoteEvent(estimator)

def EventSizeEstimator_GetSizeOfThreadForkEvent(estimator):
    c_GetSizeOfThreadForkEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfThreadForkEvent
    c_GetSizeOfThreadForkEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfThreadForkEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfThreadForkEvent(estimator)

def EventSizeEstimator_GetSizeOfThreadJoinEvent(estimator):
    c_GetSizeOfThreadJoinEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfThreadJoinEvent
    c_GetSizeOfThreadJoinEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfThreadJoinEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfThreadJoinEvent(estimator)

def EventSizeEstimator_GetSizeOfThreadTeamBeginEvent(estimator):
    c_GetSizeOfThreadTeamBeginEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfThreadTeamBeginEvent
    c_GetSizeOfThreadTeamBeginEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfThreadTeamBeginEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfThreadTeamBeginEvent(estimator)

def EventSizeEstimator_GetSizeOfThreadTeamEndEvent(estimator):
    c_GetSizeOfThreadTeamEndEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfThreadTeamEndEvent
    c_GetSizeOfThreadTeamEndEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfThreadTeamEndEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfThreadTeamEndEvent(estimator)

def EventSizeEstimator_GetSizeOfThreadAcquireLockEvent(estimator):
    c_GetSizeOfThreadAcquireLockEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfThreadAcquireLockEvent
    c_GetSizeOfThreadAcquireLockEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfThreadAcquireLockEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfThreadAcquireLockEvent(estimator)

def EventSizeEstimator_GetSizeOfThreadReleaseLockEvent(estimator):
    c_GetSizeOfThreadReleaseLockEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfThreadReleaseLockEvent
    c_GetSizeOfThreadReleaseLockEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfThreadReleaseLockEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfThreadReleaseLockEvent(estimator)

def EventSizeEstimator_GetSizeOfThreadTaskCreateEvent(estimator):
    c_GetSizeOfThreadTaskCreateEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfThreadTaskCreateEvent
    c_GetSizeOfThreadTaskCreateEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfThreadTaskCreateEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfThreadTaskCreateEvent(estimator)

def EventSizeEstimator_GetSizeOfThreadTaskSwitchEvent(estimator):
    c_GetSizeOfThreadTaskSwitchEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfThreadTaskSwitchEvent
    c_GetSizeOfThreadTaskSwitchEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfThreadTaskSwitchEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfThreadTaskSwitchEvent(estimator)

def EventSizeEstimator_GetSizeOfThreadTaskCompleteEvent(estimator):
    c_GetSizeOfThreadTaskCompleteEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfThreadTaskCompleteEvent
    c_GetSizeOfThreadTaskCompleteEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfThreadTaskCompleteEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfThreadTaskCompleteEvent(estimator)

def EventSizeEstimator_GetSizeOfThreadCreateEvent(estimator):
    c_GetSizeOfThreadCreateEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfThreadCreateEvent
    c_GetSizeOfThreadCreateEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfThreadCreateEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfThreadCreateEvent(estimator)

def EventSizeEstimator_GetSizeOfThreadBeginEvent(estimator):
    c_GetSizeOfThreadBeginEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfThreadBeginEvent
    c_GetSizeOfThreadBeginEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfThreadBeginEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfThreadBeginEvent(estimator)

def EventSizeEstimator_GetSizeOfThreadWaitEvent(estimator):
    c_GetSizeOfThreadWaitEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfThreadWaitEvent
    c_GetSizeOfThreadWaitEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfThreadWaitEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfThreadWaitEvent(estimator)

def EventSizeEstimator_GetSizeOfThreadEndEvent(estimator):
    c_GetSizeOfThreadEndEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfThreadEndEvent
    c_GetSizeOfThreadEndEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfThreadEndEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfThreadEndEvent(estimator)

def EventSizeEstimator_GetSizeOfCallingContextEnterEvent(estimator):
    c_GetSizeOfCallingContextEnterEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfCallingContextEnterEvent
    c_GetSizeOfCallingContextEnterEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfCallingContextEnterEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfCallingContextEnterEvent(estimator)

def EventSizeEstimator_GetSizeOfCallingContextLeaveEvent(estimator):
    c_GetSizeOfCallingContextLeaveEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfCallingContextLeaveEvent
    c_GetSizeOfCallingContextLeaveEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfCallingContextLeaveEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfCallingContextLeaveEvent(estimator)

def EventSizeEstimator_GetSizeOfCallingContextSampleEvent(estimator):
    c_GetSizeOfCallingContextSampleEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfCallingContextSampleEvent
    c_GetSizeOfCallingContextSampleEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfCallingContextSampleEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfCallingContextSampleEvent(estimator)

def EventSizeEstimator_GetSizeOfIoCreateHandleEvent(estimator):
    c_GetSizeOfIoCreateHandleEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfIoCreateHandleEvent
    c_GetSizeOfIoCreateHandleEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfIoCreateHandleEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfIoCreateHandleEvent(estimator)

def EventSizeEstimator_GetSizeOfIoDestroyHandleEvent(estimator):
    c_GetSizeOfIoDestroyHandleEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfIoDestroyHandleEvent
    c_GetSizeOfIoDestroyHandleEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfIoDestroyHandleEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfIoDestroyHandleEvent(estimator)

def EventSizeEstimator_GetSizeOfIoDuplicateHandleEvent(estimator):
    c_GetSizeOfIoDuplicateHandleEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfIoDuplicateHandleEvent
    c_GetSizeOfIoDuplicateHandleEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfIoDuplicateHandleEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfIoDuplicateHandleEvent(estimator)

def EventSizeEstimator_GetSizeOfIoSeekEvent(estimator):
    c_GetSizeOfIoSeekEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfIoSeekEvent
    c_GetSizeOfIoSeekEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfIoSeekEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfIoSeekEvent(estimator)

def EventSizeEstimator_GetSizeOfIoChangeStatusFlagsEvent(estimator):
    c_GetSizeOfIoChangeStatusFlagsEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfIoChangeStatusFlagsEvent
    c_GetSizeOfIoChangeStatusFlagsEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfIoChangeStatusFlagsEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfIoChangeStatusFlagsEvent(estimator)

def EventSizeEstimator_GetSizeOfIoDeleteFileEvent(estimator):
    c_GetSizeOfIoDeleteFileEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfIoDeleteFileEvent
    c_GetSizeOfIoDeleteFileEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfIoDeleteFileEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfIoDeleteFileEvent(estimator)

def EventSizeEstimator_GetSizeOfIoOperationBeginEvent(estimator):
    c_GetSizeOfIoOperationBeginEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfIoOperationBeginEvent
    c_GetSizeOfIoOperationBeginEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfIoOperationBeginEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfIoOperationBeginEvent(estimator)

def EventSizeEstimator_GetSizeOfIoOperationTestEvent(estimator):
    c_GetSizeOfIoOperationTestEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfIoOperationTestEvent
    c_GetSizeOfIoOperationTestEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfIoOperationTestEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfIoOperationTestEvent(estimator)

def EventSizeEstimator_GetSizeOfIoOperationIssuedEvent(estimator):
    c_GetSizeOfIoOperationIssuedEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfIoOperationIssuedEvent
    c_GetSizeOfIoOperationIssuedEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfIoOperationIssuedEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfIoOperationIssuedEvent(estimator)

def EventSizeEstimator_GetSizeOfIoOperationCompleteEvent(estimator):
    c_GetSizeOfIoOperationCompleteEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfIoOperationCompleteEvent
    c_GetSizeOfIoOperationCompleteEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfIoOperationCompleteEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfIoOperationCompleteEvent(estimator)

def EventSizeEstimator_GetSizeOfIoOperationCancelledEvent(estimator):
    c_GetSizeOfIoOperationCancelledEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfIoOperationCancelledEvent
    c_GetSizeOfIoOperationCancelledEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfIoOperationCancelledEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfIoOperationCancelledEvent(estimator)

def EventSizeEstimator_GetSizeOfIoAcquireLockEvent(estimator):
    c_GetSizeOfIoAcquireLockEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfIoAcquireLockEvent
    c_GetSizeOfIoAcquireLockEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfIoAcquireLockEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfIoAcquireLockEvent(estimator)

def EventSizeEstimator_GetSizeOfIoReleaseLockEvent(estimator):
    c_GetSizeOfIoReleaseLockEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfIoReleaseLockEvent
    c_GetSizeOfIoReleaseLockEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfIoReleaseLockEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfIoReleaseLockEvent(estimator)

def EventSizeEstimator_GetSizeOfIoTryLockEvent(estimator):
    c_GetSizeOfIoTryLockEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfIoTryLockEvent
    c_GetSizeOfIoTryLockEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfIoTryLockEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfIoTryLockEvent(estimator)

def EventSizeEstimator_GetSizeOfProgramBeginEvent(estimator, numberOfArguments):
    c_GetSizeOfProgramBeginEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfProgramBeginEvent
    c_GetSizeOfProgramBeginEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator), ctypes.c_uint32 ]
    c_GetSizeOfProgramBeginEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfProgramBeginEvent(estimator, numberOfArguments)

def EventSizeEstimator_GetSizeOfProgramEndEvent(estimator):
    c_GetSizeOfProgramEndEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfProgramEndEvent
    c_GetSizeOfProgramEndEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfProgramEndEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfProgramEndEvent(estimator)

def EventSizeEstimator_GetSizeOfNonBlockingCollectiveRequestEvent(estimator):
    c_GetSizeOfNonBlockingCollectiveRequestEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfNonBlockingCollectiveRequestEvent
    c_GetSizeOfNonBlockingCollectiveRequestEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfNonBlockingCollectiveRequestEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfNonBlockingCollectiveRequestEvent(estimator)

def EventSizeEstimator_GetSizeOfNonBlockingCollectiveCompleteEvent(estimator):
    c_GetSizeOfNonBlockingCollectiveCompleteEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfNonBlockingCollectiveCompleteEvent
    c_GetSizeOfNonBlockingCollectiveCompleteEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfNonBlockingCollectiveCompleteEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfNonBlockingCollectiveCompleteEvent(estimator)

def EventSizeEstimator_GetSizeOfCommCreateEvent(estimator):
    c_GetSizeOfCommCreateEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfCommCreateEvent
    c_GetSizeOfCommCreateEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfCommCreateEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfCommCreateEvent(estimator)

def EventSizeEstimator_GetSizeOfCommDestroyEvent(estimator):
    c_GetSizeOfCommDestroyEvent = conf.lib.OTF2_EventSizeEstimator_GetSizeOfCommDestroyEvent
    c_GetSizeOfCommDestroyEvent.argtypes = [ ctypes.POINTER(EventSizeEstimator) ]
    c_GetSizeOfCommDestroyEvent.restype = ctypes.c_size_t
    return c_EventSizeEstimator_GetSizeOfCommDestroyEvent(estimator)

__all__ = [
    'EventSizeEstimator',
    'EventSizeEstimator_New',
    'EventSizeEstimator_Delete',
    'EventSizeEstimator_GetDefChunkSize',
    'EventSizeEstimator_GetSizeOfTimestamp',
    'EventSizeEstimator_GetSizeOfAttributeList',
    'EventSizeEstimator_SetNumberOfStringDefinitions',
    'EventSizeEstimator_SetNumberOfAttributeDefinitions',
    'EventSizeEstimator_SetNumberOfLocationDefinitions',
    'EventSizeEstimator_SetNumberOfRegionDefinitions',
    'EventSizeEstimator_SetNumberOfGroupDefinitions',
    'EventSizeEstimator_SetNumberOfMetricDefinitions',
    'EventSizeEstimator_SetNumberOfCommDefinitions',
    'EventSizeEstimator_SetNumberOfParameterDefinitions',
    'EventSizeEstimator_SetNumberOfRmaWinDefinitions',
    'EventSizeEstimator_SetNumberOfSourceCodeLocationDefinitions',
    'EventSizeEstimator_SetNumberOfCallingContextDefinitions',
    'EventSizeEstimator_SetNumberOfInterruptGeneratorDefinitions',
    'EventSizeEstimator_SetNumberOfIoFileDefinitions',
    'EventSizeEstimator_SetNumberOfIoHandleDefinitions',
    'EventSizeEstimator_SetNumberOfLocationGroupDefinitions',
    'EventSizeEstimator_GetSizeOfBufferFlushEvent',
    'EventSizeEstimator_GetSizeOfMeasurementOnOffEvent',
    'EventSizeEstimator_GetSizeOfEnterEvent',
    'EventSizeEstimator_GetSizeOfLeaveEvent',
    'EventSizeEstimator_GetSizeOfMpiSendEvent',
    'EventSizeEstimator_GetSizeOfMpiIsendEvent',
    'EventSizeEstimator_GetSizeOfMpiIsendCompleteEvent',
    'EventSizeEstimator_GetSizeOfMpiIrecvRequestEvent',
    'EventSizeEstimator_GetSizeOfMpiRecvEvent',
    'EventSizeEstimator_GetSizeOfMpiIrecvEvent',
    'EventSizeEstimator_GetSizeOfMpiRequestTestEvent',
    'EventSizeEstimator_GetSizeOfMpiRequestCancelledEvent',
    'EventSizeEstimator_GetSizeOfMpiCollectiveBeginEvent',
    'EventSizeEstimator_GetSizeOfMpiCollectiveEndEvent',
    'EventSizeEstimator_GetSizeOfOmpForkEvent',
    'EventSizeEstimator_GetSizeOfOmpJoinEvent',
    'EventSizeEstimator_GetSizeOfOmpAcquireLockEvent',
    'EventSizeEstimator_GetSizeOfOmpReleaseLockEvent',
    'EventSizeEstimator_GetSizeOfOmpTaskCreateEvent',
    'EventSizeEstimator_GetSizeOfOmpTaskSwitchEvent',
    'EventSizeEstimator_GetSizeOfOmpTaskCompleteEvent',
    'EventSizeEstimator_GetSizeOfMetricEvent',
    'EventSizeEstimator_GetSizeOfParameterStringEvent',
    'EventSizeEstimator_GetSizeOfParameterIntEvent',
    'EventSizeEstimator_GetSizeOfParameterUnsignedIntEvent',
    'EventSizeEstimator_GetSizeOfRmaWinCreateEvent',
    'EventSizeEstimator_GetSizeOfRmaWinDestroyEvent',
    'EventSizeEstimator_GetSizeOfRmaCollectiveBeginEvent',
    'EventSizeEstimator_GetSizeOfRmaCollectiveEndEvent',
    'EventSizeEstimator_GetSizeOfRmaGroupSyncEvent',
    'EventSizeEstimator_GetSizeOfRmaRequestLockEvent',
    'EventSizeEstimator_GetSizeOfRmaAcquireLockEvent',
    'EventSizeEstimator_GetSizeOfRmaTryLockEvent',
    'EventSizeEstimator_GetSizeOfRmaReleaseLockEvent',
    'EventSizeEstimator_GetSizeOfRmaSyncEvent',
    'EventSizeEstimator_GetSizeOfRmaWaitChangeEvent',
    'EventSizeEstimator_GetSizeOfRmaPutEvent',
    'EventSizeEstimator_GetSizeOfRmaGetEvent',
    'EventSizeEstimator_GetSizeOfRmaAtomicEvent',
    'EventSizeEstimator_GetSizeOfRmaOpCompleteBlockingEvent',
    'EventSizeEstimator_GetSizeOfRmaOpCompleteNonBlockingEvent',
    'EventSizeEstimator_GetSizeOfRmaOpTestEvent',
    'EventSizeEstimator_GetSizeOfRmaOpCompleteRemoteEvent',
    'EventSizeEstimator_GetSizeOfThreadForkEvent',
    'EventSizeEstimator_GetSizeOfThreadJoinEvent',
    'EventSizeEstimator_GetSizeOfThreadTeamBeginEvent',
    'EventSizeEstimator_GetSizeOfThreadTeamEndEvent',
    'EventSizeEstimator_GetSizeOfThreadAcquireLockEvent',
    'EventSizeEstimator_GetSizeOfThreadReleaseLockEvent',
    'EventSizeEstimator_GetSizeOfThreadTaskCreateEvent',
    'EventSizeEstimator_GetSizeOfThreadTaskSwitchEvent',
    'EventSizeEstimator_GetSizeOfThreadTaskCompleteEvent',
    'EventSizeEstimator_GetSizeOfThreadCreateEvent',
    'EventSizeEstimator_GetSizeOfThreadBeginEvent',
    'EventSizeEstimator_GetSizeOfThreadWaitEvent',
    'EventSizeEstimator_GetSizeOfThreadEndEvent',
    'EventSizeEstimator_GetSizeOfCallingContextEnterEvent',
    'EventSizeEstimator_GetSizeOfCallingContextLeaveEvent',
    'EventSizeEstimator_GetSizeOfCallingContextSampleEvent',
    'EventSizeEstimator_GetSizeOfIoCreateHandleEvent',
    'EventSizeEstimator_GetSizeOfIoDestroyHandleEvent',
    'EventSizeEstimator_GetSizeOfIoDuplicateHandleEvent',
    'EventSizeEstimator_GetSizeOfIoSeekEvent',
    'EventSizeEstimator_GetSizeOfIoChangeStatusFlagsEvent',
    'EventSizeEstimator_GetSizeOfIoDeleteFileEvent',
    'EventSizeEstimator_GetSizeOfIoOperationBeginEvent',
    'EventSizeEstimator_GetSizeOfIoOperationTestEvent',
    'EventSizeEstimator_GetSizeOfIoOperationIssuedEvent',
    'EventSizeEstimator_GetSizeOfIoOperationCompleteEvent',
    'EventSizeEstimator_GetSizeOfIoOperationCancelledEvent',
    'EventSizeEstimator_GetSizeOfIoAcquireLockEvent',
    'EventSizeEstimator_GetSizeOfIoReleaseLockEvent',
    'EventSizeEstimator_GetSizeOfIoTryLockEvent',
    'EventSizeEstimator_GetSizeOfProgramBeginEvent',
    'EventSizeEstimator_GetSizeOfProgramEndEvent',
    'EventSizeEstimator_GetSizeOfNonBlockingCollectiveRequestEvent',
    'EventSizeEstimator_GetSizeOfNonBlockingCollectiveCompleteEvent',
    'EventSizeEstimator_GetSizeOfCommCreateEvent',
    'EventSizeEstimator_GetSizeOfCommDestroyEvent',
]
