/*
 * This file is part of the Score-P software (http://www.score-p.org)
 *
 * Copyright (c) 2013,
 * Technische Universitaet Dresden, Germany
 *
 * This software may be modified and distributed under the terms of
 * a BSD-style license.  See the COPYING file in the package base
 * directory for details.
 *
 */


/**
 *  @internal
 *
 *  @file
 *
 *  @brief      Callbacks for the serial case.
 */


#ifndef OTF2_INTERNAL_COLLECTIVES_SERIAL_H
#define OTF2_INTERNAL_COLLECTIVES_SERIAL_H


extern const OTF2_CollectiveCallbacks otf2_collectives_serial;


#endif /* OTF2_INTERNAL_COLLECTIVES_SERIAL_H */
