/*
 * This file is part of the Score-P software ecosystem (http://www.score-p.org)
 *
 * Copyright (c) 2020, 2022,
 * Forschungszentrum Juelich GmbH, Germany
 *
 * This software may be modified and distributed under the terms of
 * a BSD-style license. See the COPYING file in the package base
 * directory for details.
 *
 */


#include <config.h>
#include <UTILS_Mutex.h>

#define STATIC_INLINE
#include "UTILS_Mutex.inc.c"
#undef STATIC_INLINE
